FazBrowse GitHub Viewer
|
Trending
|
URL:
|
Home
Tools:
[Download Repo ZIP]
[View Raw Code]
[Original HTTPS Page]
TensorArray/src/tensor-array/core/devices.cc at master · TensorArray/TensorArray · GitHub
Uh oh!
There was an error while loading.
Please reload this page
.
TensorArray
/
TensorArray
Public
forked from
Tensor-Array/Tensor-Array
Notifications
You must be signed in to change notification settings
Fork
0
Star
0
Code
Pull requests
0
Actions
Projects
Security and quality
0
Insights
Additional navigation options
Code
Pull requests
Actions
Projects
Security and quality
Insights
Expand file tree
Breadcrumbs
TensorArray
/
src
/
tensor-array
/
core
/
devices.cc
Copy path
More file actions
More file actions
Latest commit
History
History
History
238 lines (222 loc) · 6.2 KB
Breadcrumbs
TensorArray
/
src
/
tensor-array
/
core
/
devices.cc
Copy path
File metadata and controls
238 lines (222 loc) · 6.2 KB
Raw
Copy raw file
Download raw file
Open symbols panel
Edit and raw actions
1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
/*
Copyright 2024 TensorArray-Creators
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
#
include
<
cuda_runtime.h
>
#
include
"
devices.hh
"
#
include
<
cstdio
>
#
include
<
cstdlib
>
#
include
<
cassert
>
#
include
<
cstring
>
#
include
<
mutex
>
namespace
tensor_array
{
namespace
devices
{
thread_local
Device default_dev =
DEVICE_CPU_0
;
Device&
local_device
()
{
return
default_dev;
}
void
device_memcpy
(
void
* dst, Device dst_dev,
const
void
* src, Device src_dev,
size_t
count)
{
int
temp;
if
(dst_dev.
dev_t
==
CPU
&& src_dev.
dev_t
==
CPU
)
std::memcpy
(dst, src, count);
else
if
(dst_dev.
dev_t
==
CUDA
&& src_dev.
dev_t
==
CUDA
)
assert
(
cudaMemcpyPeer
(dst, dst_dev.
index
, src, src_dev.
index
, count) == cudaSuccess);
else
if
(dst_dev.
dev_t
==
CPU
&& src_dev.
dev_t
==
CUDA
)
{
cudaError cudaStatus =
cudaGetDevice
(&temp);
cudaStatus =
cudaSetDevice
(src_dev.
index
);
cudaStatus =
cudaMemcpy
(dst, src, count, cudaMemcpyDeviceToHost);
cudaStatus =
cudaSetDevice
(temp);
}
else
if
(dst_dev.
dev_t
==
CUDA
&& src_dev.
dev_t
==
CPU
)
{
cudaError cudaStatus =
cudaGetDevice
(&temp);
cudaStatus =
cudaSetDevice
(dst_dev.
index
);
cudaStatus =
cudaMemcpy
(dst, src, count, cudaMemcpyHostToDevice);
cudaStatus =
cudaSetDevice
(temp);
}
else
{
void
* temp_data =
std::malloc
(count);
device_memcpy
(temp_data,
DEVICE_CPU_0
, src, src_dev, count);
device_memcpy
(dst, dst_dev, temp_data,
DEVICE_CPU_0
, count);
std::free
(temp_data);
}
}
void
device_memcpy
(
void
* dst, Device dst_dev,
const
void
* src, Device src_dev,
size_t
count,
void
* stream)
{
int
temp;
if
(dst_dev.
dev_t
==
CPU
&& src_dev.
dev_t
==
CPU
)
std::memcpy
(dst, src, count);
else
if
(dst_dev.
dev_t
==
CUDA
&& src_dev.
dev_t
==
CUDA
)
assert
(
cudaMemcpyPeerAsync
(dst, dst_dev.
index
, src, src_dev.
index
, count,
static_cast
<cudaStream_t>(stream)) == cudaSuccess);
else
if
(dst_dev.
dev_t
==
CPU
&& src_dev.
dev_t
==
CUDA
)
{
cudaError cudaStatus =
cudaGetDevice
(&temp);
cudaStatus =
cudaSetDevice
(src_dev.
index
);
cudaStatus =
cudaMemcpyAsync
(dst, src, count, cudaMemcpyDeviceToHost,
static_cast
<cudaStream_t>(stream));
cudaStatus =
cudaSetDevice
(temp);
}
else
if
(dst_dev.
dev_t
==
CUDA
&& src_dev.
dev_t
==
CPU
)
{
cudaError cudaStatus =
cudaGetDevice
(&temp);
cudaStatus =
cudaSetDevice
(dst_dev.
index
);
cudaStatus =
cudaMemcpyAsync
(dst, src, count, cudaMemcpyHostToDevice,
static_cast
<cudaStream_t>(stream));
cudaStatus =
cudaSetDevice
(temp);
}
else
{
void
* temp_data =
std::malloc
(count);
device_memcpy
(temp_data,
DEVICE_CPU_0
, src, src_dev, count, stream);
device_memcpy
(dst, dst_dev, temp_data,
DEVICE_CPU_0
, count, stream);
std::free
(temp_data);
}
}
void
device_memset
(
void
* dst, Device dst_dev,
int
value,
size_t
count)
{
int
temp;
if
(dst_dev.
dev_t
==
CPU
)
std::memset
(dst, value, count);
else
if
(dst_dev.
dev_t
==
CUDA
)
{
cudaError cudaStatus =
cudaGetDevice
(&temp);
cudaStatus =
cudaSetDevice
(dst_dev.
index
);
cudaStatus =
cudaMemset
(dst, value, count);
cudaStatus =
cudaSetDevice
(temp);
}
else
{
}
}
void
device_memset
(
void
* dst, Device dst_dev,
int
value,
size_t
count,
void
* stream)
{
int
temp;
if
(dst_dev.
dev_t
==
CPU
)
std::memset
(dst, value, count);
else
if
(dst_dev.
dev_t
==
CUDA
)
{
cudaError cudaStatus =
cudaGetDevice
(&temp);
cudaStatus =
cudaSetDevice
(dst_dev.
index
);
cudaStatus =
cudaMemsetAsync
(dst, value, count,
static_cast
<cudaStream_t>(stream));
cudaStatus =
cudaSetDevice
(temp);
}
else
{
}
}
void
device_CUDA_get_info
()
{
int
temp;
cudaError cudaStatus =
cudaGetDevice
(&temp);
cudaDeviceProp prop;
cudaGetDeviceProperties
(&prop,
0
);
printf
(
"
Device name: %s
\n
"
, prop.
name
);
printf
(
"
Memory Clock Rate (KHz): %d
\n
"
,
prop.
memoryClockRate
);
printf
(
"
Memory Bus Width (bits): %d
\n
"
,
prop.
memoryBusWidth
);
printf
(
"
Peak Memory Bandwidth (GB/s): %f
\n\n
"
,
2.0
* prop.
memoryClockRate
* (prop.
memoryBusWidth
/
8
) /
1.0e6
);
}
}
}
void
*
operator
new
(
size_t
count, tensor_array::devices::Device dev)
{
int
temp;
void
* m_alloc_dat;
switch
(dev.
dev_t
)
{
case
tensor_array::devices::
CPU
:
m_alloc_dat =
std::malloc
(count);
break
;
case
tensor_array::devices::
CUDA
:
{
cudaError_t cuda_status =
cudaGetDevice
(&temp);
cuda_status =
cudaSetDevice
(dev.
index
);
cuda_status =
cudaMalloc
(&m_alloc_dat, count);
cuda_status =
cudaSetDevice
(temp);
}
break
;
default
:
throw
0
;
break
;
}
return
m_alloc_dat;
}
void
*
operator
new
(
size_t
count, tensor_array::devices::Device dev,
void
* stream)
{
int
temp;
void
* m_alloc_dat;
switch
(dev.
dev_t
)
{
case
tensor_array::devices::
CPU
:
m_alloc_dat =
std::malloc
(count);
break
;
case
tensor_array::devices::
CUDA
:
{
cudaError_t cuda_status =
cudaGetDevice
(&temp);
cuda_status =
cudaSetDevice
(dev.
index
);
cuda_status =
cudaMallocAsync
(&m_alloc_dat, count,
static_cast
<cudaStream_t>(stream));
cuda_status =
cudaSetDevice
(temp);
}
break
;
default
:
throw
0
;
break
;
}
return
m_alloc_dat;
}
void
operator
delete
(
void
* data, tensor_array::devices::Device dev)
{
int
temp;
switch
(dev.
dev_t
)
{
case
tensor_array::devices::
CPU
:
std::free
(data);
break
;
case
tensor_array::devices::
CUDA
:
{
cudaGetDevice
(&temp);
cudaSetDevice
(dev.
index
);
cudaFree
(data);
cudaSetDevice
(temp);
}
break
;
default
:
throw
0
;
break
;
}
}
void
operator
delete
(
void
* data, tensor_array::devices::Device dev,
void
* stream)
{
int
temp;
switch
(dev.
dev_t
)
{
case
tensor_array::devices::
CPU
:
std::free
(data);
break
;
case
tensor_array::devices::
CUDA
:
{
cudaGetDevice
(&temp);
cudaSetDevice
(dev.
index
);
cudaFreeAsync
(data,
static_cast
<cudaStream_t>(stream));
cudaSetDevice
(temp);
}
break
;
default
:
throw
0
;
break
;
}
}
Back
|
FazBrowse Home
|
New Git URL